/*
	This file is part of B-cubed.

	Copyright (C) 2009, 2010, 2011, Edward Rosten and Susan Cox

	B-cubed is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 3.0 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU General Public License     
	along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

#ifndef MULTISPOT5_H
#define MULTISPOT5_H
#include <vector>
#include <string>
#include <fstream>
#include <iostream>
#include <tr1/memory>
#include <cvd/image.h>
#include <cvd/byte.h>
#include <TooN/TooN.h>
#include <TooN/so2.h>

#include "utility.h"
#include "mt19937.h"


/// Graphics class for FittingSpots.
/// This abstraction prevents FitSpots from depending on and graphics library.
/// The functions are tied very heavily to the internals of FitSpots.
class FitSpotsGraphics
{
	public:
		///Initialize graphics
		///@param size Size of window for display
		virtual void init(CVD::ImageRef size)=0;

		///Draw a bunch of stuff
		///@param spots List of spots to draw
		///@param im Background image
		///@param box Bounding box of region
		///@param N Spot to highlight
		///@param s Extra spot to draw as a cross
		virtual void draw_krap(const std::vector<TooN::Vector<4> >& spots, const CVD::Image<CVD::byte>& im, const BBox& box, int N, TooN::Vector<4> s = TooN::Ones * 1e99)=0;

		///Swap buffers if double buffered
		virtual void swap()=0;

		///Draw the pixel mask in an (r,g,b,a) tuple colour
		///@param pix mask
		///@param r red
		///@param g green
		///@param b blue
		///@param a alpha
		virtual void draw_pixels(const std::vector<CVD::ImageRef>& pix, float r, float g, float b, float a=1)=0;

		///Draw a bounding box
		///@param bbox Box corners
		virtual void draw_bbox(const BBox& bbox)=0;

		///Draw a cross
		///@param p Position of cross
		///@param size Size to draw cross
		virtual void glDrawCross(const TooN::Vector<2>& p, int size=3)=0;

		///Desctructor
		virtual ~FitSpotsGraphics();
};

std::auto_ptr<FitSpotsGraphics> null_graphics();

///Null struct thrown if a parse error is encountered when trying to load a log file.
struct LogFileParseError
{
	///@param s error message to set
	LogFileParseError(const std::string &s)
	:what(s)
	{}
	
	/// variable holding the parse error error message
	std::string what;
};

void fit_spots_historic(const CVD::BasicImage<double>& log_ratios, const std::vector<CVD::Image<float> >& ims, std::vector<CVD::ImageRef> pixels, FitSpotsGraphics& g);

///Internal state (excluding fixed settings) which represents the entire
///internal state of spot fitting. Used to restart from interruptions.
struct StateParameters{
	std::tr1::shared_ptr<MT19937> rng;    ///< Random number generator state
	std::vector<TooN::Vector<4> > spots;  ///< Spots positions 
	int pass;                             ///< Pass number
	int iteration;                        ///< Iteration number
	std::vector<CVD::ImageRef> pixels;    ///< Area for analysis
};

StateParameters generate_state_parameters_ye_olde(const CVD::BasicImage<double>& log_ratios, const std::vector<CVD::Image<float> >& ims, std::vector<CVD::ImageRef> pixels);

void fit_spots_new(const std::vector<CVD::Image<float> >& ims, StateParameters& p, std::ofstream& save_spots, FitSpotsGraphics&);

StateParameters parse_log_file(std::istream& in);
#endif
